#!/usr/bin/perl

#
# FBAT_PLOT.PL :  Prepare  FBAT output for displaying with gnuplot
# Syntax    :  perl fbat_plot.pl [-h] [-clean] [options]
# Infiles   :  map.<chr> fbat_log.<chr>
# Outfiles  :  myplot.gp, chr_line, add.plt, dom.plt and rec.plt
# Modified  : 2-May-2005	FR

use Cwd;
# use strict;

print "\n\n";
print " * * * * * * * * * * * FBAT_PLOT.PL v1.0  * * * * * * * * * * * *\n";
print " * * * * * * * * View Fbat results with GNUPLOT   * * * * * * * *\n";

my $deltmp=1;		# delete temporary files, 1- yes, 0 - no
my $hline=1;		# print horizontal lines,
my $legend=1;		# print legend,
my $label=1;		# print chr labels
my $tit=1;

for (my $i=0;$i<=$#ARGV;$i++) {
	if ($ARGV[$i] eq "-h" ) { &Help;}
	elsif ($ARGV[$i] eq "-v" ) { &Help;}
	elsif ($ARGV[$i] =~ "-nodelete" ) { $deltmp=0;}
	elsif ($ARGV[$i] =~ "-clean" ) { &Clean;}
	elsif ($ARGV[$i] =~ "-nohline" ) {$hline=0;}
	elsif ($ARGV[$i] eq "-nt" ) {$tit=0;}
	elsif ($ARGV[$i] =~ "-nolegend" ) {$legend=0;}
	elsif ($ARGV[$i] =~ "-nolabel" ) {$label=0;}
	else {print "Argument: $ARGV[$i]\n";}
}

print "\nRead ...\n";

my @xsums=();
my $xsum = 0.0;
my $lastxsum=0.;
my $xmax=0.0;
my $xmin = 0;

my $ymax = 1;
my $ymin = 0.000001;

my $pfad = getcwd();
my $datum = scalar localtime;
my %chrlabel=();
my $status=0;
my $in_file="";
my $mapfile="";
my %gpos=();			# chromosomal position cM
my $coord=0; 			# genome position cM
my $lastmarker="XXX";
my @notplot=();			# pvalues < 0.000001  are in fbat 0.000000
my $chr="";
my $hit=0.001;	# threshold for hitlist (selects p-values < $hit)
my $thr=0.1;	# gnuplot shows only pvalues smaller than threshold

#######################################    M A I N   #############################															
open (out1,">$pfad/add.plt") || die "could not open add.plt :$!\n";
open (out2,">$pfad/dom.plt") || die "could not open dom.plt :$!\n";
open (out3,">$pfad/rec.plt") || die "could not open rec.plt :$!\n";
open (out4,">$pfad/gen.plt") || die "could not open gen.plt :$!\n";
open (out5,">$pfad/hits.out") || die "could not open hits.out :$!\n";

	
foreach my $dir (c01,c02,c03,c04,c05,c06,c07,c08,c09,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21,c22,c23) {
	if (-d $dir) {
		$status=0;
		$xmax=0.0;
		$chr=$dir;
		$chr=~ s/c//;

		$in_file = "${dir}/fbat_log\.$chr";	# no sets of markers, one outfile per chromosome
		if (-f "$in_file") {
			$mapfile="${dir}/map.${chr}";	  
			&ReadMap;  
			&ReadPval();
		}
		else { 
			my @set=glob("${dir}/fbat_log_*.$chr"); # moving window with sets of markers
			my $nr=$#set+1;
			foreach my $i ( 1..$nr) {		
				$in_file = "${dir}/fbat_log_${i}\.$chr";
				if (-f "$in_file") {
					$mapfile="${dir}/map_${i}.${chr}";	  
					&ReadMap;  
					&ReadPval(); 
				}
			}
		} 
		if ($status == 1) {
			$xsum = $xsum + $xmax;
			my $xpos=(($lastxsum + $xsum)/2);
			my $var=$dir; $var =~ s/c0//;$var =~ s/c//;
			if ($var > 9){ $xpos -=30;}
			else { $xpos -=5;}
			if ($var eq "23") {$var = "X";}
			if ($var == "24") {$var = "XY";}
			$chrlabel{$var}=$xpos;
			$lastxsum=$xsum;
			print out1 "\n";
			print out2 "\n";
			print out3 "\n";
			print out4 "\n";
			print out5 "\n";
			
		}
		push (@xsums,$xsum);
	}
	else { print "$dir not existing \n";
	}
} # end foreach

close (out1);
close (out2);
close (out3);
close (out4);
close (out5);

foreach (@notplot) { print "can not plot $_\n";}
	 
&print_chr_line;
&pr_batch;

# print "\nYou can now view results by \"gnuplot myplot.gp\" \n\n";
system ("gnuplot myplot.gp");

if ($deltmp == 1) { &Clean ();}


################################ R E A D M A P  ################################
sub ReadMap {
	%gpos=();
	open (IN, "$mapfile") || die "could not open $mapfile :$!\n";
	while (<IN>){
		next if /^#/;
		next if /^$/;
		chomp;
		my @v = split(' ',$_);	
		$gpos{$v[2]}=$v[1];
	}
	close (IN);
}

#########################   R E A D  P - V A L U E S   #########################  

sub ReadPval {
	my @v=();
	my $model="false";
	my $lastmarker="XXX";
	my $lastpval="";
	open (IN, "$in_file") || die "could not open $in_file :$!\n";
	print "File: $in_file\n"; 
	$status =1;
	while (<IN>){
		chomp;
		next if /fam\#/;
		@v = split(' ',$_);
		if ($v[0] eq "Total") {$model="false";next;}
		if ($#v < 7) {next;}
		if ($v[5] eq "additive\;"){$model="add";next;}
		if ($v[5] eq "dominant\;") {$model="dom";next;}
		if ($v[5] eq "recessive\;") {$model="rec";next;}
		if ($v[5] eq "genotype\;") {$model="gen";next;}

		if ($model ne "false") {
			if (! $gpos{$v[0]}) {print "No genetic position for $v[0]\n";}
			$coord=$xsum + $gpos{$v[0]};
			if ($gpos{$v[0]} > $xmax) { $xmax = $gpos{$v[0]};}  # chromosome length
			if ($v[$#v] < $ymin) { push (@notplot,"$chr $gpos{$v[0]} $_"); $v[$#v] = $ymin;}
		}
		
		if ($model eq "add") {
			if ($v[0] eq $lastmarker) {
				if (($v[$#v] > $thr ) && ($lastpval > $thr)) {next;}
				if ($v[$#v] < $lastpval) { printf(out1 "%f %s\n",$coord, $v[$#v]);}
				else {printf(out1 "%f %s\n",$coord, $lastpval);}
			}

		}
		elsif ($model eq "dom") {
			if ($v[0] eq $lastmarker) {
				if (($v[$#v] > $thr ) && ($lastpval > $thr)) {next;}
				if ($v[$#v] < $lastpval) { printf(out2 "%f %s\n",$coord, $v[$#v]);}
				else {printf(out2 "%f %s\n",$coord, $lastpval);}
			}			
		}
		elsif ($model eq "rec") {
			if ($v[0] eq $lastmarker) {
				if (($v[$#v] > $thr ) && ($lastpval > $thr)) {next;}
				if ($v[$#v] < $lastpval) { printf(out3 "%f %s\n",$coord, $v[$#v]);}
				else {printf(out3 "%f %s\n",$coord, $lastpval);}
			}
		}			
		elsif ($model eq "gen") {
			if ($v[0] eq $lastmarker) {
				if (($v[$#v] > $thr ) && ($lastpval > $thr)) {next;}
				if ($v[$#v] < $lastpval) { printf(out4 "%f %s\n",$coord, $v[$#v]);}
				else {printf(out4 "%f %s\n",$coord, $lastpval);}
			}
		}
		else {;}
		
		$lastmarker=$v[0];
		$lastpval=$v[$#v];
		if ($v[$#v] < $hit) {
			print out5 "$chr $gpos{$v[0]}\t$_\n";
		}
		
	} # end while
	close (IN);
	
} ## end ReadPval


######################   P R I N T  C H R _ L I N E S   ######################
sub print_chr_line {

	open (OUT,"> chr_line.plt") || die "could not open chr_line.plt :$!\n";

	if ($hline=1){
		foreach my $i ("0.1", "0.01", "0.001", "0.0001", "0.00001" ) {	# horizontale Linien bei TDT p-value = 0.1, 0.01,...
			print OUT "0 $i\n$xsum $i\n\n";
		}
	}

	foreach (@xsums){				# vertikale Linien Chromosomenende 1,2,3,4,...22,x,xy
		print OUT "$_ $ymin \n$_ $ymax \n\n";         
	}
	close (OUT);
}
###-----------------------  P R I N T  G N U P L O T  B A T C H   -----------------------###
sub pr_batch {
	my $titel="";

	
	open (out5,"> myplot.gp") || die "could not open myplot.gp :$!\n";

	print out5 "set logscale y\n";
#	print out5 "set autoscale\n";
	print out5 "set xlabel \"cM\"\n";
	my $ypos = "0.000002";
#	my $ypos = $ymax + (($ymax-$ymin)/22);
	if ($label == 1) {	
		foreach (keys %chrlabel) {print out5 "set label \"$_\" at $chrlabel{$_}\,$ypos\n";}
	}
# 	print out5 "set size 1,0.8\n";
    print out5 "set key below \n";
	if ($legend == 0){ print out5 "set nokey\n";}
	print out5 "set terminal postscript color\n";

	print out5 "set ylabel \"p-value\"\n";

	$titel = "$pfad - FBAT all models p-value - $datum";
	if ($tit ==1) {print out5 "set title \"$titel\"\n";}
	print out5 "set output \"Fbat_mix.ps\"\n";
	print out5 "plot \[$xmin\:$xsum\] \[$ymax\:$ymin\] \'chr_line.plt\' notitle with lines lt 0, \'add.plt\' title 'add' with points lt 4, \'dom.plt\' title 'dom' with points lt 2, \'rec.plt\' title 'rec' with points lt 3, \'gen.plt\' title 'gen' with points lt 1\n";		
	print out5 "set terminal windows\n";
	print out5 "replot\n";
	print out5 "\npause -1 \"Hit return to continue\"\n\n";

	$titel = "$pfad - FBAT additive - $datum";
	print out5 "set title \"$titel\"\n";
	print out5 "set terminal postscript color\n";
	print out5 "set output \"Fbat_additive.ps\"\n";
	print out5 "plot \[$xmin\:$xsum\] \[$ymax\:$ymin\] \'chr_line.plt\' notitle with lines lt 0, \'add.plt\' title 'add' with points lt 3\n";
	print out5 "set terminal windows\n";
	print out5 "replot\n";
	print out5 "\npause -1 \"Hit return to continue\"\n\n";

	$titel = "$pfad - FBAT dominant - $datum";
	print out5 "set title \"$titel\"\n";
	print out5 "set terminal postscript color\n";
	print out5 "set output \"Fbat_dominant.ps\"\n";
	print out5 "plot \[$xmin\:$xsum\] \[$ymax\:$ymin\] \'chr_line.plt\' notitle with lines lt 0, \'dom.plt\' title 'dom' with points lt 3\n";
	print out5 "set terminal windows\n";
	print out5 "replot\n";
	print out5 "\npause -1 \"Hit return to continue\"\n\n";

	$titel = "$pfad - FBAT recessive - $datum";
	print out5 "set title \"$titel\"\n";
	print out5 "set terminal postscript color\n";
	print out5 "set output \"Fbat_recessive.ps\"\n";
	print out5 "plot \[$xmin\:$xsum\] \[$ymax\:$ymin\] \'chr_line.plt\' notitle with lines lt 0, \'rec.plt\' title 'rec' with points lt 3\n";
	print out5 "set terminal windows\n";
	print out5 "replot\n";
	print out5 "\npause -1 \"Hit return to continue\"\n\n";
	
	$titel = "$pfad - FBAT genotype - $datum";
	print out5 "set title \"$titel\"\n";
	print out5 "set terminal postscript color\n";
	print out5 "set output \"Fbat_genotype.ps\"\n";
	print out5 "plot \[$xmin\:$xsum\] \[$ymax\:$ymin\] \'chr_line.plt\' notitle with lines lt 0, \'gen.plt\' title 'gen' with points lt 3\n";
	print out5 "set terminal windows\n";
	print out5 "replot\n";
	print out5 "\npause -1 \"Hit return to continue\"\n\n";
	
	close (out5);

} ## end pr_batch
					

#################################   H E L P   #################################  
sub Help {

print " * * * * * * (c) 2005 Created by Franz Rueschendorf  * * * * * * * \n";
print " * * * * * * * * * Max-Delbrueck-Center, Berlin  * * * * * * * * * \n";
print " * * * * * * * * * email: fruesch\@mdc-berlin.de  * * * * * * * * * \n\n";
print "  Usage        : perl [path]fbat_plot.pl [<options>]\n";
print "  Infiles      : fbat_log.<chr>\n";
print "  Outfiles     : myplot.gp, chr_line.plt, add.plt, dom.plt, rec.plt and gen.plt \n" ;
print "  View results : gnuplot myplot.gp\n\n";
exit 0;
}

################################   C L E A N   ################################  
#
# removes files in the current folder with the extension *.plt and *.gp
#
sub Clean { 
unlink glob("*.plt");
unlink glob("*.gp");
exit 0;
}

############################# END Fbat_PLOT.PL  #########################
