#!/usr/bin/perl

#
# merlin_lod.pl :  Prepares Merlin output for displaying with gnuplot
# Syntax    :  merlin_lod.pl [-h] [-clean] [-hlod]
# Infiles   :  merlin.out (default)
# Outfiles  :  myLOD.gp, chr_line.plt, lod.plt, hlod.plt, merlin_hlod.ps and merlin_lod.ps
# 
# Created 12-Aug-2005 FR 

use Cwd;
use strict;

my $showhlod=0;
my $nodelete=0;
my $hline=1;
my $legend=1;
my $label=1;
my $tit=1;

for (my $i=0;$i<=$#ARGV;$i++) {
	if ($ARGV[$i] eq "-h" ) { &Help;}
	elsif ($ARGV[$i] eq "-v" ) { &Help;}
	elsif ($ARGV[$i] eq "-nd" ) { $nodelete=1;}
	elsif ($ARGV[$i] =~ "-clean" ) { &Clean;}
	elsif ($ARGV[$i] eq "-nhline" ) {$hline=0;}
	elsif ($ARGV[$i] eq "-nl" ) {$legend=0;}
	elsif ($ARGV[$i] eq "-nlabel" ) {$label=0;}
	elsif ($ARGV[$i] eq "-nt" ) {$tit=0;}
	elsif ($ARGV[$i] eq "-hlod" ) {$showhlod=1;}
	else {print "Argument: $ARGV[$i]\n";}
}

print "\n\n";
print " * * * * * * * * * * * *  MERLIN_LOD.PL v1.1 * * * * * * * * * * * * *\n";
print " * * * * * * * * * View Merlin results with GNUPLOT  * * * * * * * * *\n\n";
print "Please wait ...\n\n";

my $maxlines = 999999; 	# Maximum number of lines (input-file)

my $xsum = 0.0;
my $xmax = 2;
my $xmin = 0;
my $ymax = 2;
my $ymin = -3;
my $yhlodmax=0;

my $lastxsum=0.;
my @xsums = ();
my %chrlabel=();
my $pfad = cwd();
my $infile="";
my $datum = scalar localtime;

open (OUT1,"> lod.plt") || die "could not open lod.plt :$!\n";
open (OUT4,"> hlod.plt") || die "could not open hlod.plt :$!\n";

foreach my $dir qw(c01 c02 c03 c04 c05 c06 c07 c08 c09 c10 c11 c12 c13 c14 c15 c16 c17 c18 c19 c20 c21 c22 c23) {
	if (-f "$dir/merlin.out") {
		$infile = "$dir/merlin.out";
		print "File: $infile "; 
		&readLOD;

		my $xpos=(($lastxsum + $xsum)/2);
		my $var=$dir; $var =~ s/c0//;$var =~ s/c//;
		if ($var eq "23"){$var = "X";}
		if ($var eq "24"){$var = "XY";}
		if ($var > 9){ $xpos -=30;}
		else { $xpos -=5;}
		$chrlabel{$var}=$xpos;	  
		$lastxsum=$xsum;

		push (@xsums,$xsum);
	}
	else { print "$dir/merlin.out not existing \n"; }
} # end foreach
close (OUT1);
close (OUT4);

&print_chr_line;
&pr_batch;

print "\nYou can now view results by \"gnuplot myLOD.gp\" \n\n";
system "gnuplot  myLOD.gp";

if ($nodelete ==0) {&clean;}


#######################  R E A D    L O D - S C O R E S  #######################

sub readLOD {
	open (IN, "$infile") || die "could not open $infile :$!\n";

	my $jetzt = $maxlines;
	$xmax = 0.0;
	my $hit=0;

	while (<IN>){
		my @feld =	split(' ',$_);
		if (($feld[0] =~ /Parametric/) and ($feld[1] =~ /Analysis/))   { 
			print "\t$feld[3] $feld[4]\n";
			$hit=1;
		}
		if ($feld[0] =~ /Skipping/) {print "\t$_\n";}

		if (($feld[0] =~ /POSITION/) && ($feld[3] =~ /HLOD/))  { $jetzt = $.;}

		if ($#feld < 1) { $jetzt = $maxlines;}

		if ($. > ($jetzt)) {
		printf(OUT1 "%f %s\n", ($feld[0] + $xsum), $feld[1]);
		printf(OUT4 "%f %s\n", ($feld[0] + $xsum), $feld[3]);
		if ($feld[0] > $xmax) { $xmax = $feld[0];}  
		if ($feld[1] > $ymax) { $ymax = $feld[1];}
		if ($feld[3] > $yhlodmax) { $yhlodmax = $feld[3];}  
		}
	} # end while
	if ($hit == 0) {print "  no analysis\n";}
	$xsum = $xsum + $xmax;
	printf OUT1 "\n";
	printf OUT4 "\n";

  close (IN);

} ## end readLOD


######################   P R I N T  C H R _ L I N E S   ######################
sub print_chr_line {
if ($showhlod == 1) { $ymax = $yhlodmax;}
	
	$ymax += 1;			# y-achse auf ganze LOD-zahl bringen
	my @buff =	split('\.',$ymax);
	$ymax=$buff[0];

	open (OUT3,"> chr_line.plt") || die "could not open chr_line.plt :$!\n";

	for (my $i=$ymin+1; $i<$ymax;$i++) {	# Linien bei LODs = 1,2,3,4,..
		print OUT3 "0 $i\n$xsum $i\n\n";
	}
	foreach (@xsums){	# Linien Chromosomenende 1,2,3,4,...22,x,xy
		print OUT3 "$_ $ymin \n$_ $ymax \n\n";         
	}
	close (OUT3);
}


###-----------------------  P R I N T  G N U P L O T  B A T C H   -----------------------###
sub pr_batch {
	
	my $titel = "$pfad - Merlin parametric analysis - $datum";
	open (OUT2,"> myLOD.gp") || die "could not open myLOD.gp :$!\n";

	print OUT2 "set autoscale\n";
 	if ($tit == 1) { print OUT2 "set title \"$titel\\n\"\n" ;}
	print OUT2 "set xlabel \"cM\"\n";
	if ($legend == 0) { print OUT2 "set nokey\n";}
	if ($showhlod == 1) {print OUT2 "set ylabel \"HLOD\"\n";}
	else {print OUT2 "set ylabel \"LOD\"\n";}
	my $ypos = $ymax + (($ymax-$ymin)/25);
	foreach (keys %chrlabel) {print OUT2 "set label \"$_\" at $chrlabel{$_}\,$ypos\n";}
	print OUT2 "set size 1,0.9\n";
	print OUT2 "set terminal postscript color solid 11\n";
	if ($showhlod == 1) {
		print OUT2 "set output \"merlin_hlod.ps\"\n";
		print OUT2 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'chr_line.plt\' notitle with lines lt 0, \'hlod.plt\'  title 'HLOD'with lines lt 3\n";
	}	else {
		print OUT2 "set output \"merlin_lod.ps\"\n";
		print OUT2 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'chr_line.plt\' notitle with lines lt 0, \'lod.plt\' title 'LOD' with lines lt 3\n";
	}
	print OUT2 "set terminal windows\n";
	print OUT2 "replot\n";
	print OUT2 "\npause -1 \"Hit return to continue\"\n\n";

	close (OUT2);
}


##################################  H E L P  ############################
sub Help {
print "\n\n";
print " * * * * * * * * * * * * * MERLIN_LOD.PL v1.0  * * * * * * * * * * * *\n";
print " * * * * * * * * * View Merlin results with GNUPLOT  * * * * * * * * *\n";
print " * * * * * * * *  Created by Franz Rueschendorf  2005  * * * * * * * *\n";
print " * * * * * * * * * *  Max-Delbrueck-Center, Berlin * * * * * * * * * *\n";
print " * * * * * * * * * *  email: fruesch\@mdc-berlin.de * * * * * * * * * *\n\n";
print "Usage        : perl merlin_lod.pl [-h] [-clean] [-hlod]\n";
print "Infiles      : merlin.out (default)\n";
print "Outfiles     : myLOD.gp, lod.plt, hlod.plt, chr_line.plt, merlin_hlod.ps\n" ;
print "View results : gnuplot myLOD.gp\n\n";
exit 0;
}

sub clean { 
unlink glob("*.plt");
unlink glob("*.gp");
exit 0;
}

### end merlin_LOD.pl
