#!/usr/bin/perl

#
# merlin_npl.pl :  Prepares Merlin output for displaying with gnuplot
# Syntax    :  merlin_npl.pl [-h] [-clean] [-lod]
# Infiles   :  merlin.out (default)
# Outfiles  :  mynpl.gp, chr_line.plt, npl.plt and merlin_npl.ps
# 
# Created 15-Jan-2003 FR 
# Modified 1-Aug-2005 FR

use Cwd;
use strict;

my $showlod=0;
my $nodelete=0;
my $hline=1;
my $legend=1;
my $label=1;
my $tit=1;

for (my $i=0;$i<=$#ARGV;$i++) {
	if ($ARGV[$i] eq "-h" ) { &Help;}
	elsif ($ARGV[$i] eq "-v" ) { &Help;}
	elsif ($ARGV[$i] eq "-nd" ) { $nodelete=1;}
	elsif ($ARGV[$i] =~ "-clean" ) { &Clean;}
	elsif ($ARGV[$i] eq "-nhline" ) {$hline=0;}
	elsif ($ARGV[$i] eq "-nl" ) {$legend=0;}
	elsif ($ARGV[$i] eq "-nlabel" ) {$label=0;}
	elsif ($ARGV[$i] eq "-nt" ) {$tit=0;}
	elsif ($ARGV[$i] eq "-lod" ) {$showlod=1;}
	elsif ($ARGV[$i] eq "-info" ) {$showlod=2;}
	else {print "Argument: $ARGV[$i]\n";}
}

print "\n\n";
print " * * * * * * * * * * * *  MERLIN_NPL.PL v1.1 * * * * * * * * * * * * *\n";
print " * * * * * * * * * View Merlin results with GNUPLOT  * * * * * * * * *\n\n";
print "Please wait ...\n\n";

my $maxlines = 999999; 	# Maximum number of lines (input-file)

my $xsum = 0.0;
my $xmin = 0.;

my $ymax = 2.;
my $ymin = -1;
my $lastxsum=0.;
my @xsums = ();
my %chrlabel=();
my $pfad = cwd();
my $datum = scalar localtime;
my $infile="";

open (OUT1,"> npl.plt") || die "could not open npl.plt :$!\n";
open (OUT4,"> lod.plt") || die "could not open lod.plt :$!\n";
open (OUT5,"> info.plt") || die "could not open info.plt :$!\n";

# foreach my $dir qw(c23) { # for one chromosome edit this line
foreach my $dir qw(c01 c02 c03 c04 c05 c06 c07 c08 c09 c10 c11 c12 c13 c14 c15 c16 c17 c18 c19 c20 c21 c22 c23 c24) {
	$infile = "$dir/merlin.out";
	if (-f "$infile") {
		print "File: $infile ";
		if ($showlod == 2) { &readinfo;} 
		else {&readnpl;}

		my $xpos=(($lastxsum + $xsum)/2);
		my $var=$dir; $var =~ s/c0//;$var =~ s/c//;
		if ($var eq "23"){$var = "X";}
		if ($var eq "24"){$var = "XY";}
		if ($var > 9){ $xpos -=30;}
		else { $xpos -=5;}
		$chrlabel{$var}=$xpos;	  
		$lastxsum=$xsum;

		push (@xsums,$xsum);
	}
	else { print "$dir/merlin.out not existing \n"; }
} # end foreach
close (OUT1);
close (OUT4);
close (OUT5);

&print_chr_line;
&pr_batch;

print "\nYou can now view results by \"gnuplot mynpl.gp\" \n\n";
system "gnuplot  mynpl.gp";

if ($nodelete ==0) {&Clean;}

#######################  R E A D   N P L - S C O R E S  #######################

sub readnpl {
	open (IN, "$infile") || die "could not open $infile :$!\n";

	my $jetzt = $maxlines;
	my $xmax = 0.0;
	my $hit=0;

	while (<IN>){
		my @feld =	split(' ',$_);
		if (($feld[0] =~ /Pheno/) and ($feld[1] =~ /TRAIT/))   { 
			my $score=$feld[2];
			print "  score: $score\n";
			$hit=1;
		}
		if ($feld[0] =~ /Skipping/) {print "\t$_\n";}

		if ($feld[0] =~ /max/)   { $jetzt = $. ;}

		if ($#feld < 1) { $jetzt = $maxlines;}

		if ($. > ($jetzt)) {
		printf(OUT1 "%f %s\n", ($feld[0] + $xsum), $feld[1]);
		printf(OUT4 "%f %s\n", ($feld[0] + $xsum), $feld[4]);
		if ($feld[0] > $xmax) { $xmax = $feld[0];}  
		if ($feld[1] > $ymax) { $ymax = $feld[1];}
		}
	} # end while
	if ($hit == 0) {print "  no analysis\n";}
	$xsum = $xsum + $xmax;
	printf OUT1 "\n";
	printf OUT4 "\n";

	close (IN);

} ## end readnpl

######################  R E A D   I N F O  C O N T E N T  ######################

sub readinfo {
	open (IN, "$infile") || die "could not open $infile :$!\n";

	my $jetzt = $maxlines;
	my $xmax = 0.0;
	my $hit=0;

	while (<IN>){
		my @feld =	split(' ',$_);
		if (($feld[0] =~ /Position/) and ($feld[1] =~ /Info/))   { 
			print "  read info\n";
			$hit=1;
			$jetzt = $.;
		}
		if ($feld[0] =~ /Skipping/) {print "\t$_\n";}

		if ($#feld < 1) { $jetzt = $maxlines;}

		if ($. > ($jetzt)) {
		printf(OUT5 "%f %s\n", ($feld[0] + $xsum), $feld[1]);
		if ($feld[0] > $xmax) { $xmax = $feld[0];}  
		if ($feld[1] > $ymax) { $ymax = $feld[1];}
		}
	} # end while

	if ($hit == 0) {print "  no information content analysis\n";}
	$xsum = $xsum + $xmax;
	print OUT5 "\n";
	close (IN);

} ## end readinfo


######################   P R I N T  C H R _ L I N E S   ######################
sub print_chr_line {
	$ymax += 1;			# y-achse auf ganze NPL-zahl bringen
	my @buff =	split('\.',$ymax);
	$ymax=$buff[0];

	open (OUT3,"> chr_line.plt") || die "could not open chr_line.plt :$!\n";

	if ($showlod == 2) {
		my @var = qw( 0.2 0.4 0.6 0.8);
		foreach my $i (@var) {	# Linien bei = 0.2, 0.4,...
			print OUT3 "0 $i\n$xsum $i\n\n";
		}
	}
	else {
		for (my $i=1; $i<$ymax;$i++) {	# Linien bei LODs = 1,2,3,4,..
			print OUT3 "0 $i\n$xsum $i\n\n";
		}
	}

	pop @xsums;
	if ($showlod == 2) {
		foreach (@xsums){	# Linien Chromosomenende 1,2,3,4,...22,x,xy
			print OUT3 "$_ 0 \n$_ 1 \n\n";         
		}
	}
	else {		
		foreach (@xsums){	# Linien Chromosomenende 1,2,3,4,...22,x,xy
			print OUT3 "$_ $ymin \n$_ $ymax \n\n";         
		}
	}
	close (OUT3);
}


###-----------------------  P R I N T  G N U P L O T  B A T C H   -----------------------###
sub pr_batch {
	my $ypos = 1.04;
	my $titel = "$pfad - Merlin non-parametric analysis - $datum";
	open (OUT2,"> mynpl.gp") || die "could not open mynpl.gp :$!\n";

	print OUT2 "set autoscale\n";
 	if ($tit == 1) {print OUT2 "set title \"$titel\\n\"\n";}
	if ($legend == 0) { print out5 "set nokey\n";}
	print OUT2 "set xlabel \"cM\"\n";

	if ($showlod == 1) {print OUT2 "set ylabel \"LOD*\"\n";}
	elsif ($showlod == 2) {print OUT2 "set ylabel \"Information Content\"\n";}
	else {print OUT2 "set ylabel \"Zmean/LOD*\"\n";}

	if ($showlod < 2) { $ypos = $ymax + (($ymax-$ymin)/25); }
	foreach (keys %chrlabel) {print OUT2 "set label \"$_\" at $chrlabel{$_}\,$ypos\n";}
	
	print OUT2 "set size 1,0.9\n";
	print OUT2 "set terminal postscript color solid 12\n";
	if ($showlod == 1) {
		print OUT2 "set output \"merlin_lod.ps\"\n";
		print OUT2 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'chr_line.plt\' notitle with lines lt 0, \'lod.plt\' title 'LOD\*' with lines lt 1\n";
	}	
	elsif ($showlod == 2) {
		print OUT2 "set output \"merlin_info.ps\"\n";
		print OUT2 "plot \[$xmin\:$xsum\] \[0\:1\]  \'chr_line.plt\' notitle with lines lt 0, \'info.plt\' title 'Info' with lines lt 1\n";
	}
	else {
		print OUT2 "set output \"merlin_npl.ps\"\n";
		print OUT2 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'chr_line.plt\' notitle with lines lt 0, \'npl.plt\' title 'Z-mean' with lines lt 3,\'lod.plt\' title 'LOD\*' with lines lt 1\n";
	}
	print OUT2 "set terminal windows\n";
	print OUT2 "replot\n";
	print OUT2 "\npause -1 \"Hit return to continue\"\n\n";

	close (OUT2);
}


##################################  H E L P  ############################
sub Help {
print "\n\n";
print " * * * * * * * * * * * * * MERLIN_NPL.PL v1.1  * * * * * * * * * * * *\n";
print " * * * * * * * * * View Merlin results with GNUPLOT  * * * * * * * * *\n";
print " * * * * * * * *  Created by Franz Rueschendorf  2003  * * * * * * * *\n";
print " * * * * * * * * * *  Max-Delbrueck-Center, Berlin * * * * * * * * * *\n";
print " * * * * * * * * * *  email: fruesch\@mdc-berlin.de * * * * * * * * * *\n\n";
print "Usage        : perl merlin_npl.pl [-h] [-clean] [-lod]\n";
print "Infiles      : merlin.out (default)\n";
print "Outfiles     : mynpl.gp, npl.plt, lod.plt, chr_line.plt, merlin_npl.ps\n" ;
print "View results : gnuplot mynpl.gp\n\n";
exit 0;
}

sub Clean { 
unlink glob("*.plt");
unlink glob("*.gp");
exit 0;
}

### end merlin_npl.pl
