#!/usr/bin/perl

#
# SIB_TDT_PLOT.pl :  Prepare ASPEXs SIB_TDT output for displaying with gnuplot
# Syntax    :  perl SIB_TDT_plot.pl [-h] [-clean] [-mat]
# Infiles   :  <sib_tdt_*.out> <map_*.chr>
# Outfiles  :  myplot.gp, chr_line, sum.plt, max.plt and dsum.plt
# Modified  : 19-Sept-2004	FR

use Cwd;
# use strict;

print "\n\n";
print " * * * * * * * * * * * SIB_TDT_PLOT.PL v1.0  * * * * * * * * * * * *\n";
print " * * * * * * * * View SIB_TDT results with GNUPLOT   * * * * * * * *\n";

my $deltmp=1;		# delete temporary files, 1- yes, 0 - no
my $hline=1;		# print horizontal lines,
my $mat=0;			# print maternal results 1-maternal, 0-paternal
my $legend=1;		# print legend,
my $label=1;		# print chr labels
my $tit=1;
my $pmax=0;			# Plot 0-max and sum value, 1-max, 2-sum  


for (my $i=0;$i<=$#ARGV;$i++) {
	if ($ARGV[$i] eq "-h" ) { &Help;}
	elsif ($ARGV[$i] eq "-v" ) { &Help;}
	elsif ($ARGV[$i] =~ "-nd" ) { $deltmp=0;}
	elsif ($ARGV[$i] =~ "-clean" ) { &Clean;}
	elsif ($ARGV[$i] =~ "-nohline" ) {$hline=0;}
	elsif ($ARGV[$i] eq "-nt" ) {$tit=0;}
	elsif ($ARGV[$i] =~ "-nl" ) {$legend=0;}
	elsif ($ARGV[$i] =~ "-nolabel" ) {$label=0;}
	elsif ($ARGV[$i] =~ /-max/ ) {$pmax=1;}
	elsif ($ARGV[$i] =~ /-sum/ ) {$pmax=2;}
	elsif ($ARGV[$i] =~ /-pat/ ) {$mat=0;}
	elsif ($ARGV[$i] =~ /-mat/ ) {$mat=1;}
	else {print "Argument: $ARGV[$i]\n";}
}

print "\nRead ...\n";

my @xsums=();
my $xsum = 0.0;
my $lastxsum=0.;
my $xmax=0.0;
my $xmin = 0;

my $ymax = 0.1;
my $ymin = 0.000001;

my $pfad = getcwd();
my $datum = scalar localtime;
my %chrlabel=();
my $status=0;
my $in_file="";
my $mapfile="";
my %gpos=();			# chromosomal position cM
my $coord=0; 			# genome position cM
my $lastmarker="XXX";
my $chr="";
my $hit=0.01;	# threshold for hitlist (selects p-values < $hit)

#######################################    M A I N   #############################															
open (out1,">$pfad/sum.plt") || die "could not open sum.plt :$!\n";
open (out2,">$pfad/max.plt") || die "could not open max.plt :$!\n";
open (out3,">$pfad/dsum.plt") || die "could not open dsum.plt :$!\n";
open (out4,">$pfad/hits.out") || die "could not open hits.out :$!\n";

	
foreach my $dir (c01,c02,c03,c04,c05,c06,c07,c08,c09,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21,c22,c23,cx,c24,cxy) {
	if (-d $dir) {
		$status=0;
		$xmax=0.0;
		$chr=$dir;
		$chr=~ s/c//;

		$in_file = "${dir}/sib_tdt\.out";	# no sets of markers, one outfile per chromosome
		if (-f "$in_file") {
			$mapfile="${dir}/map.${chr}";	  
			&ReadMap;  
			&ReadPval();
		}
		else { 
			my @set=glob("${dir}/sib_tdt_*.out"); # moving window with sets of markers
			my $nr=$#set+1;
			foreach my $i ( 1..$nr) {		
				$in_file = "${dir}/sib_tdt_${i}\.out";
				if (-f "$in_file") {
					$mapfile="${dir}/map_${i}.${chr}";	  
					&ReadMap;  
					&ReadPval(); 
				}
			}
		} 
		if ($status == 1) {
			$xsum = $xsum + $xmax;
#			print "XSUM: $xsum\t Koord: $coord\n";
			my $xpos=(($lastxsum + $xsum)/2);
			my $var=$dir; $var =~ s/c0//;$var =~ s/c//;
			if ($var > 9){ $xpos -=30;}
			else { $xpos -=5;}
			if ($var eq "23") {$var = "X";}
			if ($var == "24") {$var = "XY";}
			$chrlabel{$var}=$xpos;
			$lastxsum=$xsum;
			print out1 "\n";
			print out2 "\n";
			print out4 "\n";
			
		}
		push (@xsums,$xsum);
	}
	else { print "$dir not existing \n";
	}
} # end foreach

close (out1);
close (out2);
close (out3);
close (out4);

&print_chr_line;
&pr_batch;

# print "\nYou can now view results by \"gnuplot myplot.gp\" \n\n";
system ("gnuplot myplot.gp");

if ($deltmp == 1) { &Clean ();}


################################ R E A D M A P  ################################
sub ReadMap {
	%gpos=();
	open (IN, "$mapfile") || die "could not open $mapfile :$!\n";
	while (<IN>){
		next if /^#/;
		next if /^$/;
		chomp;
		my @v = split(' ',$_);	
		$gpos{$v[2]}=$v[1];
	}
	close (IN);
}

#########################   R E A D  P - V A L U E S   #########################  

sub ReadPval {
	my @v=();
	
	open (IN, "$in_file") || die "could not open $in_file :$!\n";
	print "File: $in_file\n"; 
	$status =1;
	while (<IN>){
		chomp;
		@v = split(' ',$_);
#		if ($v[0] eq $lastmarker) {next;}
		if ($. > 3) {
			if (! $gpos{$v[0]}) {print "No genetic position for $v[0]\n";}
			$coord=$xsum + $gpos{$v[0]};
			if ($gpos{$v[0]} > $xmax) { $xmax = $gpos{$v[0]};}  # chromosome length
			if ($mat == 0) {
				printf(out1 "%f %s\n",$coord, $v[2]);
				printf(out2 "%f %s\n",$coord, $v[4]);
				if ($v[2] > $ymax) { $ymax = $v[2];}
				if ($v[4] > $ymax) { $ymax = $v[4];}
			}
			else {
				printf(out1 "%f %s\n",$coord, $v[6]);
				printf(out2 "%f %s\n",$coord, $v[8]);
				if ($v[6] > $ymax) { $ymax = $v[6];}
				if ($v[8] > $ymax) { $ymax = $v[8];}
			}
			if (($v[2] < $hit) || ($v[4] < $hit)||($v[6] < $hit) || ($v[8] < $hit)) { print out4 "$chr $gpos{$v[0]}\t$_\n";}
		} # end if
		$lastmarker=$v[0];
	} # end while
	close (IN);
	
} ## end ReadPval

######################   P R I N T  C H R _ L I N E S   ######################
sub print_chr_line {

#	$ymax += 1;						# y-achse auf ganze Hchi2-Zahl bringen
#	my @buff =	split('\.',$ymax);		# round to integer
#	$ymax=$buff[0];

	open (out3,"> chr_line") || die "could not open chr_line :$!\n";

	if ($hline=1){
		foreach my $i ("0.01", "0.001", "0.0001", "0.00001" ) {	# horizontale Linien bei TDT p-value = 0.1, 0.01,...
			print out3 "0 $i\n$xsum $i\n\n";
		}
	}
	pop @xsums;
	foreach (@xsums){				# vertikale Linien Chromosomenende 1,2,3,4,...22,x,xy
		print out3 "$_ $ymin \n$_ 0.1 \n\n";         
	}
	close (out3);
}
###-----------------------  P R I N T  G N U P L O T  B A T C H   -----------------------###
sub pr_batch {
	my $titel="";
	if ($mat == 1) {$titel = "$pfad - SIB_TDT p-value maternal - $datum";}
	else {$titel = "$pfad - SIB_TDT p-value paternal - $datum";}
	
	open (out5,"> myplot.gp") || die "could not open myplot.gp :$!\n";

	print out5 "set logscale y\n";
	print out5 "set nokey\n";
	if ($tit ==1) {print out5 "set title \"$titel\"\n";}
	print out5 "set xlabel \"cM\"\n";
	my $ypos = "0.0000005";
#	my $ypos = $ymax + (($ymax-$ymin)/22);
	if ($label == 1) {	
		foreach (keys %chrlabel) {print out5 "set label \"$_\" at $chrlabel{$_}\,$ypos\n";}
	}
	print out5 "set size 1,0.5\n";
#    print out5 "set key below \n";
	if ($legend == 0){ print out5 "set nokey\n";}
	print out5 "set terminal postscript color\n";
	print out5 "set ylabel \"p-value\"\n";
	if ($mat == 1) {
		if ($pmax==1) {print out5 "set output \"SIB_TDT_maternal_max.ps\"\n";}
		elsif ($pmax==2) {print out5 "set output \"SIB_TDT_maternal_sum.ps\"\n";}
		else {print out5 "set output \"SIB_TDT_maternal.ps\"\n";}
	}
	else {
		if ($pmax==1) {print out5 "set output \"SIB_TDT_paternal_max.ps\"\n";}
		elsif ($pmax==2) {print out5 "set output \"SIB_TDT_paternal_sum.ps\"\n";}
		else {print out5 "set output \"SIB_TDT_paternal.ps\"\n";}
	}		

	if ($pmax==1) {print out5 "plot \[$xmin\:$xsum\] \[0.1\:$ymin\] \'max.plt\' title 'max' with points, \'chr_line\' notitle with lines lt 0\n";}
	elsif ($pmax==2) {print out5 "plot \[$xmin\:$xsum\] \[0.1\:$ymin\] \'sum.plt\' title 'sum' with points lt 3, \'chr_line\' notitle with lines lt 0\n";}
	else {print out5 "plot \[$xmin\:$xsum\] \[0.1\:$ymin\] \'max.plt\' title 'max' with points lt 1, \'sum.plt\' title 'sum' with points lt 3, \'chr_line\' notitle with lines lt 0\n";}

	print out5 "set terminal windows\n";
	print out5 "replot\n";
	print out5 "\npause -1 \"Hit return to continue\"\n\n";

	close (out5);

} ## end pr_batch
					

#################################   H E L P   #################################  
sub Help {

print " * * * * * * * Created by Franz Rueschendorf 2005  * * * * * * * * \n";
print " * * * * * * * * * Max-Delbrueck-Center, Berlin  * * * * * * * * * \n";
print " * * * * * * * * * email: fruesch\@mdc-berlin.de  * * * * * * * * * \n\n";
print "  Usage        : perl [path]sib_tdt_plot.pl [<options>]\n";
print "  Infiles      : sib_tdt_<x>.out\n";
print "  Outfiles     : myplot.gp, chr_line, sum.plt, max.plt and dsum.plt \n" ;
print "  View results : gnuplot myplot.gp\n\n";
exit 0;
}

################################   C L E A N   ################################  
#
# removes files in the current folder with the extension *.plt *.gp and chr_line
#
sub Clean { 
unlink glob("*.plt");
unlink glob("*.gp");
unlink "chr_line";
exit 0;
}

############################# END SIB_TDT_PLOT.PL  #########################
