#!/usr/bin/perl

#
# gh_snp_qtl_plot.pl :  Prepare Genehunter output for displaying with gnuplot
# Syntax    :  perl gh_snp_qtl_plot.pl [-h] [-clean] [-hlod] [trad-he|em-he|ml|ml_ndv|np]
# Infiles   :  <Argument1> (default)
# Outfiles  :  myplot.gp, chr_line z.plt and lod.plt
# Modified  : 19-Sept-2004	FR

use Cwd;
# use strict;

print "\n\n";
print " * * * * * * * * * * * *  GH_SNP_QTL_PLOT.PL v1.0  * * * * * * * * * * * *\n";
print " * * * * * * * *  View Genehunter results with GNUPLOT * * * * * * * *\n";

if ($#ARGV < 0) {&Help;}

my $sel="np"; 	# selection of analysis	[trad-he|em-he|ml|ml_ndv|np]
my $nodelete=0;	# delete temporary files
my $hline=1;	# show horizontal lines;	
my $legend=1;	# show legend
my $title=1;	# show titel
my $lod=1;		# column of LODscore
my $z=2;		# column of a second plot variable
my $t1="LOD";	# name of line1 for legend
my $t2="Z";		# name of line2 for legend
my $lines=1;	# plot one or two lines


for (my $i=0;$i<=$#ARGV;$i++) {
	if ($ARGV[$i] eq "-h" ) { &Help;}
	elsif ($ARGV[$i] eq "-v" ) { &Help;}
	elsif ($ARGV[$i] =~ "-nd" ) { $nodelete=1;}
	elsif ($ARGV[$i] =~ "-clean" ) { &Clean;}
	elsif ($ARGV[$i] =~ "-nohline" ) {$hline=0;}
	elsif ($ARGV[$i] =~ "-nl" ) {$legend=0;}
	elsif ($ARGV[$i] =~ "-nt" ) {$title=0;}
	elsif ($ARGV[$i] eq "-l2" ) {$lines=2;}
	elsif ($ARGV[$i] =~ "trad-he" ) { $sel="trad-he";$lod=2;$z=1;$t1="LOD";$t2="beta";}
	elsif ($ARGV[$i] =~ "em-he" ) { $sel="em-he";$lod=2;$z=1;$t1="LOD";$t2="beta";}
	elsif ($ARGV[$i] =~ "ml" ) { $sel="ml";$lod=1;$z=4;$t1="LOD";$t2="SigSq2";}
	elsif ($ARGV[$i] =~ "ml_ndv" ) { $sel="ml_ndv";$lod=1;$z=4;$t1="LOD";$t2="SigSq2";}
	elsif ($ARGV[$i] =~ "np" ) { $sel="np";$lod=1;$z=1;$t1="Z-score";$t2="LOD";}
	else {print "Argument: $ARGV[$i]\n\n";&Help;}
}

print "\nRead ...\n";

my @xsums=();
my $xsum = 0.0;
my $lastxsum=0.;
my $xmin = 0;

my $ymax = 2;
my $ymin = -1;

my $pfad = getcwd();
my $datum = scalar localtime;
my %chrlabel=();
my $status=0;		# found analysis on this chromosome 
my $infile="";

#######################################    M A I N   #############################															
open (out1,">$pfad/lod.plt") || die "could not open lod.plt :$!\n";
open (out2,">$pfad/z.plt") || die "could not open z.plt :$!\n";


	
foreach my $dir (c01,c02,c03,c04,c05,c06,c07,c08,c09,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21,c22,c23,cx,c24,cxy) {
	if (-d $dir) {
		$status=0;
		my $chr=$dir;
		print "$dir";
		$chr=~ s/c//;
		$infile = "${dir}/${sel}\.out";	# no sets of markers, one outfile per chromosome
		if (-f "$infile") {	
			&ReadLod();
		}
		else { 
			my @set=glob("${dir}/${sel}_*.out"); # moving window with sets of markers
			my $nr=$#set+1;
			foreach my $i ( 1..$nr) {		
				$infile = "${dir}/${sel}_${i}\.out";
				if (-f "$infile") {	  
					&ReadLod(); 
				}
			}
		} 
		if ($status == 1) {
			my $xpos=(($lastxsum + $xsum)/2);
			my $var=$dir; $var =~ s/c0//;$var =~ s/c//;
			if ($var > 9){ $xpos -=30;}
			else { $xpos -=5;}
			if ($var eq "23") {$var = "X";}
			if ($var == "24") {$var = "XY";}
			$chrlabel{$var}=$xpos;
			$lastxsum=$xsum;
		}
		push (@xsums,$xsum);
		print "\n";
	}
	else { print "$dir not exists\n";
	}
} # end foreach

close (out1);
close (out2);
&print_chr_line;
&pr_batch;

# print "\nYou can now view results by \"gnuplot myplot.gp\" \n\n";
system ("gnuplot myplot.gp");

if ($nodelete == 0) { &Clean ();}

#################################   R E A D L O D   #################################  

sub ReadLod {
	my @v=();
	my $xmax = 0.0;
	my $stop=0;
	open (IN, "$infile") || die "could not open $infile :$!\n";
	print " $infile"; 
	$status =1;
	while (<IN>){
		chomp;
		@v = split(' ',$_);
		if (($. == 2) && ($#v < 0)) {next;}
		if ($#v < 0 ){$stop=1;}
		if (($. > 1) && ($stop == 0)){
			printf(out1 "%f %s\n", ($v[0] + $xsum), $v[$lod]);
			if ($lines ==2) {printf(out2 "%f %s\n", ($v[0] + $xsum), $v[$z]);}
			if ($v[0] > $xmax) { $xmax = $v[0];}  
			if ($v[$lod] > $ymax) { $ymax = $v[$lod];}
			if ($lines ==2) { if ($v[$z] > $ymax) { $ymax = $v[$z];}}
		}
	} # end foreach
	close (IN);
	$xsum = $xsum + $xmax;
} ## end ReadLod

######################   P R I N T  C H R _ L I N E S   ######################
sub print_chr_line {

#	$xsum = 3700;		# for a fixed axle in every picture
	$ymax += 1;						# y-achse auf ganze HLOD-Zahl bringen
	my @buff =	split('\.',$ymax);		# round to integer
	$ymax=$buff[0];

	open (out3,"> chr_line.plt") || die "could not open chr_line.plt :$!\n";

	if ($hline=1){
		for (my $i= $ymin +1; $i<$ymax;$i++) {	# horizontale Linien bei LODs = 1,2,3,4,..
			print out3 "0 $i\n$xsum $i\n\n";
		}
	}
	pop @xsums;
	foreach (@xsums){				# vertikale Linien Chromosomenende 1,2,3,4,...22,x,xy
		print out3 "$_ $ymin \n$_ $ymax \n\n";         
	}
	close (out3);
}
###-----------------------  P R I N T  G N U P L O T  B A T C H   -----------------------###
sub pr_batch {

	my $name=uc($sel);
	my $titel = "$pfad - Genehunter $name  - $datum";
	
	open (out5,"> myplot.gp") || die "could not open myplot.gp :$!\n";

	print out5 "set autoscale\n";
	if ($title == 1) { print out5 "set title \"$titel\"\n";}
	print out5 "set xlabel \"cM\"\n";
	my $ypos = $ymax + (($ymax-$ymin)/22);
	foreach (keys %chrlabel) {print out5 "set label \"$_\" at $chrlabel{$_}\,$ypos\n";}
	print out5 "set size 1,0.9\n";
#    print out5 "set key below \n";
	if ($legend == 0){ print out5 "set nokey\n";}
	print out5 "set terminal postscript color solid\n";
	print out5 "set output \"gh_$sel.ps\"\n";

	if ($lines == 2) {
		print out5 "set ylabel \"$t1/$t2\"\n";
		print out5 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]   \'chr_line.plt\' notitle with lines lt 0, \'z.plt\' title \"$t2\" with lines lt 3, \'lod.plt\' title \"$t1\" with lines lt 1\n";
	}
	else {
		print out5 "set ylabel \"$t1\"\n";
		print out5 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'chr_line.plt\' notitle with lines lt 0, \'lod.plt\'  title \"$t1\" with lines lt 1\n";
	}
	print out5 "set terminal windows\n";
	print out5 "replot\n";
	print out5 "\npause -1 \"Hit return to continue\"\n\n";

	close (out5);

} ## end pr_batch
					

#################################   H E L P   #################################  
sub Help {

print " * * * * * * * Created by Franz Rueschendorf 27-May-2005 * * * * * * * \n";
print " * * * * * * * * * * Max-Delbrueck-Center, Berlin  * * * * * * * * * * \n";
print " * * * * * * * * * * email: fruesch\@mdc-berlin.de  * * * * * * * * * * \n\n";
print "  Usage        : perl [path]gh_snp_qtl_plot.pl [<options>] [trad-he|em-he|ml|ml_ndv|np]\n";
print "  Options      :\n";
print "       trad-he    Traditional Haseman-Elston\n";
print "       em-he      EM Haseman-Elston\n";
print "       ml         ML variance\n";
print "       ml_ndv     NO DOM VAR\n";
print "       np         nonparametric\n";
print "       -nd        do not delete temp. files\n";
print "       -nt        show no titel\n";
print "       -nl        show no legend\n";

print "  Infiles      : <qtl_file>.out\n";
print "  Outfiles     : myplot.gp, chr_line.plt, lod.plt and z.plt \n" ;
print "  View results : gnuplot myplot.gp\n\n";
exit 0;
}

################################   C L E A N   ################################  
#
# removes files in the current folder with the extensions *.plt and *.gp
#
sub Clean { 
unlink glob("*.plt");
unlink glob("*.gp");
exit 0;
}

### end GH_SNP_QTL_PLOT.PL
