#!/usr/bin/perl

#
# sw_snp_lod.pl :  Prepare Simwalk2 Lodscores for displaying with gnuplot
# for use with a moving window
# Syntax    :  perl sw_snp_lod.pl [-h] [-clean] [<extension>]
# Infiles   :  <simwalks_score.out> 
# Outfiles  :  swlod.gp,chr_line.plt and lod.plt
# Modified 26-Sept-2005 FR

use Cwd;
use strict;
use File::Basename;

################################   V A R I A B L E S   ##################################

my $maxlines = 999999; 	# Maximum number of lines (input-file)

my $xsum = 0.0;
my $xmin = 0;
my $ymax = 3;
my $ymin = -2;
my @xsums=();
my $lastxsum=0.;

my $pfad=cwd();
my $datum = scalar localtime;
my @map=();
my $mapfile="";
my $set=0;
my %chrlabel=();
my $postfix="";
my $chr="";
my $infile="";
my ($firstlocus,$lastlocus);

my $hline=1;

######################################  M A I N  #######################################
print "\n\n";
print " * * * * * * * * * * * * *  SW_SNP_LOD.PL v1.0  * * * * * * * * * * * * * \n";
print " * * * * * * * * * *  View SIMWALK2 LODs with GNUPLOT * * * * * * * * * * \n\n";

if ($ARGV[0] =~ "-h" ) { eval version;}
if ($ARGV[0] =~ "-v" ) { eval version;}
if ($ARGV[0] =~ "-clean" ) { eval clean;}
if ($#ARGV == "0") {$postfix = $ARGV[0];} else {$postfix = "ALL";}

print "Working ...\n\n";
my $titel = "$pfad - SW2 LODs Family: $postfix - $datum";

open (out4,"> lod.plt") || die "could not open lod.plt :$!\n";

foreach $chr ("01","02","03","04","05","06","07","08","09","10","11","12","13","14","15","16","17","18","19","20","21","22") {
	
	if ( -d "c$chr") {
		print "\n";
		$set=0;
		my $sw_run=0;
		my @maps=glob("c${chr}/map_*.$chr");
		foreach (@maps){
			my $basename=basename($_);
			my @feld =	split('_',$basename);
			if ($set < $feld[1]) {$set=$feld[1];}
		}
		
		foreach my $i ( 1..$set) {
			$infile = "c${chr}/SCORE-${chr}_${i}\.${postfix}";
			$mapfile="c${chr}/map_${i}\.${chr}";
			&get_locus_names;
			if ( -f "$infile") {&readlod;$sw_run=1;}
			else { print "\n";}
		} 
####
		if ($sw_run==1) {		# chromosome label
			my $xpos=(($lastxsum + $xsum)/2);
			my $var=$chr; $var =~ s/c0//;$var =~ s/c//;
			if ($var eq "23"){$var = "X";}
			if ($var > 9){ $xpos -=30;}
			else { $xpos -=5;}
			$chrlabel{$var}=$xpos;
			$lastxsum=$xsum;
		}
####
		push (@xsums,$xsum);
		print out4 "\n";  
		        
    }
	else { print "Chromosome $chr not exists \n"; }
} # end foreach

close (out4);

&print_chr_line;
&pr_batch;
print "\nYou can now view results by \"gnuplot swlod.gp\" \n\n";
system "gnuplot swlod.gp";
&clean;

#########################################  S U B s  ##########################################

sub readlod {
  open (infile, "$infile") || die "could not open $infile :$!\n";

  my  $jetzt = $maxlines;
  my $xmax = 0.0;
  print " : $infile   \n"; 

  while (<infile>){
  	next if /^$/;
  	next if /^\#/;
	chop();
	s/,//g;		# new simwalk format is not useful
	my @feld =	split(' ',$_);

    if ($feld[0] eq $firstlocus) { $jetzt = $.;}
    if ($feld[0] eq $lastlocus) { $jetzt = $maxlines;}
    if (($. > $jetzt) && ($#feld >= 2)){
		printf(out4 "%7.4f  %s\n", ($feld[0] + $xsum), $feld[1]);
		if ($feld[0] > $xmax) { $xmax = $feld[0];}  
		if ($feld[1] > $ymax) { $ymax = $feld[1];}
    }  
  } # end while

  $xsum = $xsum + $xmax;
  printf out4 "\n";

  close (infile);

} ## end readlod

######################   P R I N T  C H R _ L I N E S   ######################
sub print_chr_line {

#	$xsum = 3700;		# for a fixed axle in every picture
	$ymax += 1;						# y-achse auf ganze HLOD-Zahl bringen
	my @buff =	split('\.',$ymax);		# round to integer
	$ymax=$buff[0];
	
	open (OUT,"> chr_line.plt") || die "could not open chr_line.plt :$!\n";

	if ($hline=1){
		for (my $i=-1; $i<$ymax;$i++) {	# horizontale Linien bei LODs = 1,2,3,4,..
			print OUT "0 $i\n$xsum $i\n\n";
		}
	}
	pop @xsums;
	foreach (@xsums){	# vertikale Linien Chromosomenende 1,2,3,4,...22,x,xy
		print OUT "$_ $ymin \n$_ $ymax \n\n";         
	}
	close (OUT);
}
###-----------------------  P R I N T  G N U P L O T  B A T C H   -----------------------###
sub pr_batch {

#	$xsum = 3600;
	open (out5,"> swlod.gp") || die "could not open swlod.gp :$!\n";

	print out5 "set autoscale\n";
	print out5 "set title \"$titel\\n\"\n" ;
	print out5 "set xlabel \"cM\"\n";
	print out5 "set ylabel \"LOD score\"\n";
	my $ypos = $ymax + (($ymax-$ymin)/14);
	foreach (keys %chrlabel) {print out5 "set label \"$_\" at $chrlabel{$_}\,$ypos\n";}
	print out5 "set size 1,0.9\n";
	print out5 "set terminal postscript color solid\n";
	print out5 "set output \"sw_snp_lod_$postfix.ps\"\n";
	print out5 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]   \'chr_line.plt\' notitle with lines lt 0, \'lod.plt\' title 'LOD' with lines lt 1\n";
	print out5 "set terminal windows\n";
	print out5 "replot\n";
	print out5 "\npause -1 \"Hit return to continue\"\n\n";

	close (out5);

} ## end pr_batch
																
###------------------   G E T  F I R S T and L A S T  L O C U S   ----------------------###
sub get_locus_names {
	print "Mapfile $mapfile  ";
	my $first=0;
	open (infile2, "$mapfile") || die "could not open Mapfile $mapfile :$!\n";
	while (<infile2>){
  		next if /^$/;
  		next if /^\#/;
  		next if /cM/;
  		chop();
  		my @feld =	split(' ',$_);
  		if ($#feld > 1){
  			if ($first==0) { $firstlocus = $feld[2];$first=1;}
  			$lastlocus = $feld[2];
		}
	}
	close (infile2);
	if (length($firstlocus) > 8) { $firstlocus=substr($firstlocus,(length($firstlocus)-8),8);}
	if (length($lastlocus) > 8) { $lastlocus=substr($lastlocus,(length($lastlocus)-8),8);}
} # end last_locus


###------------------------------------- V E R S I O N ---------------------------------###

sub version {
print "\n\n";
print " * * * * * *  Created by Franz Rueschendorf  1-Mar-2004 * * * * * * \n";
print " * * * * * * * *  Max-Delbrueck-Center, Berlin  * * * * * * * * * * \n";
print " * * * * * * * * *  email: fruesch\@mdc-berlin.de  * * * * * * * * * \n";
print "\n     Usage        : perl sw_snp_lod.pl [-h] [-clean] [<extension>]\n";
print "     Infiles      : SCORE-<nn>\.<mmm>\n";
print "     Outfiles     : swlod.gp, chr_line.plt lod.plt \n" ;
print "     View results : gnuplot swlod.gp\n\n";
exit 0;
}


###-------------------------------------- C L E A N  ---------------------------------###

sub clean { 
unlink glob("*.plt");
unlink glob("*.gp");
exit 0;
}

### end sw_snp_lod.pl

############################################  E N D   ##########################################
